import sys

from simtk.openmm import app
from simtk.openmm.openmm import Platform
import simtk.openmm as mm
import simtk.unit as u

from miniomm.reporters import *
from miniomm.namdbin import NAMDBin
from miniomm.namdxsc import *


_cachedPdb = {}
def get_pdb(n):
    if n not in _cachedPdb:
        _cachedPdb[n] = app.PDBFile(n)
    return _cachedPdb[n]



def getBanner():
    return """
            _         _                              
 _ __ ___  (_) _ __  (_)  ___   _ __ ___   _ __ ___  
| '_ ` _ \ | || '_ \ | | / _ \ | '_ ` _ \ | '_ ` _ \ 
| | | | | || || | | || || (_) || | | | | || | | | | |
|_| |_| |_||_||_| |_||_| \___/ |_| |_| |_||_| |_| |_|
                                                     
A minimalistic OpenMM MD frontend.   
https://github.com/giorginolab/miniomm
"""





def parse_boxsize_units(txt):
    box = [float(x) for x in txt.split(" ")]
    boxa = mm.Vec3(box[0], 0., 0.) * u.angstrom
    boxb = mm.Vec3(0., box[1],  0.) * u.angstrom
    boxc = mm.Vec3(0., 0., box[2]) * u.angstrom
    return (boxa, boxb, boxc)


def get_box_size(inp):
    if 'extendedsystem' in inp:
        print("Reading box size from "+inp.extendedsystem)
        (boxa, boxb, boxc) = parse_xsc_units(inp.extendedsystem)
    elif 'boxsize' in inp:
        print("Using boxsize from input string "+inp.boxsize)
        (boxa, boxb, boxc) = parse_boxsize_units(inp.boxsize)
    else:
        print("Last resort: PDB CRYST1...")
        try:
            import warnings
            print(f"Reading positions from PDB: ")
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
            pdb = get_pdb(inp.coordinates)
            (boxa, boxb, boxc) = pdb.topology.getPeriodicBoxVectors()
        except:
            raise ValueError("Failed to load CRYST1 information")
    print("Using this cell:\n   " + str(boxa) +
          "\n   " + str(boxb) + "\n   " + str(boxc))
    return (boxa, boxb, boxc)


def get_coords(inp):
    if 'bincoordinates' in inp:
        print(f"Reading positions from NAMDBin: "+inp.bincoordinates)
        coords = NAMDBin(inp.bincoordinates).getPositions()
    else:
        import warnings
        print(f"Reading positions from PDB: ")
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            pdb = get_pdb(inp.coordinates)
        coords = pdb.positions
    return coords


def plumed_parser(fn):
    # Hack to workaround https://github.com/openmm/openmm-plumed/pull/27
    # Joins continuation lines and strips comments
    out = []
    continuing = False
    with open(fn) as f:
        for l in f:
            l = l.strip()
            if "#" in l:
                l = l[:l.find("#")]  # Strip comments
            dots = l.find("...")
            if not continuing:
                if dots == -1:
                    out.append(l)
                else:
                    out.append(l[:dots])
                    continuing = True
            else:
                if dots == -1:
                    out[-1] = out[-1] + " " + l
                else:
                    out[-1] = out[-1] + " " + l[:dots]
                    continuing = False
    return "\n".join(out)


def every(T, t):
    f = T/t
    if not f.is_integer():
        raise ValueError(f"{T} is not a multiple of {t}")
    return int(f)


def get_best_platform():
    num = Platform.getNumPlatforms()
    pp_s = {}
    for i in range(num):
        pp = Platform.getPlatform(i)
        pn = pp.getName()
        ps = float(pp.getSpeed())
        pp_s[pn] = ps

    so = sorted(pp_s.items(), key=lambda x: x[1], reverse=True)
    sel = "->"
    for i in so:
        print(sel, i[0], i[1])
        sel = "  "
    return so[0][0]


def check_openmm():
    version = Platform.getOpenMMVersion()
    plugindir = Platform.getDefaultPluginsDirectory()
    print(" OpenMM details:")
    print("  Version     : OpenMM " + str(version))
    print("  Plugin dir  : " + str(plugindir))

    # Try loading the plugins and checking for errors
    Platform.loadPluginsFromDirectory(plugindir)
    errs = Platform.getPluginLoadFailures()
    if len(errs):
        print("Some errors were found loading plugins. Some platforms may not be available: \n")
    for x in errs:
        print(x)


def add_reporters(simulation, trajectory_file, log_every, save_every,
                  total_steps, continuing, checkpoint_file):
    print(
        f"Reporting every {log_every} steps and checkpointing on {checkpoint_file} every {save_every} steps.")
    op = "append" if continuing else "write"
    print(f"Will {op} to trajectory file {trajectory_file}.\n")

    basename="output"
    fp=open(f"{basename}.log", "a" if continuing else "w")
    simulation.reporters.append(app.DCDReporter(trajectory_file, save_every,
                                                append=continuing, enforcePeriodicBox=False))
    simulation.reporters.append(app.CheckpointReporter(checkpoint_file,
                                                       save_every))
    simulation.reporters.append(StdoutLogReporter(log_every, total_steps))
    simulation.reporters.append(app.StateDataReporter(fp,
                                                      log_every,
                                                      step=True,
                                                      time=True,
                                                      potentialEnergy=True,
                                                      kineticEnergy=True,
                                                      totalEnergy=True,
                                                      temperature=True,
                                                      volume=True,
                                                      progress=True,
                                                      remainingTime=True,
                                                      speed=True,
                                                      totalSteps=total_steps,
                                                      separator='\t'))



# Simple-minded reader for NAMD files

"""Reader for NAMD binary coordinate (and velocities) format."""


import numpy as np
import simtk.unit as u
from simtk.unit.quantity import Quantity

# self.data is a Numpy array in Angstroms

class NAMDBin:

    def __init__(self, init_data):
        if type(init_data) == str:
            self.read_file(init_data)
        else:
            self.read_data(init_data)

    def read_file(self, filename):
        with open(filename, 'rb') as namdbin:
            self.n_atoms = np.fromfile(namdbin, dtype=np.int32, count=1)[0]
            coord_double = np.fromfile(namdbin,
                                       dtype=np.float64,
                                       count=self.n_atoms * 3)
        self.data = coord_double.reshape(self.n_atoms, 3)

    def read_data(self, pos):
        X = pos.value_in_unit(u.angstrom)
        self.n_atoms = X.shape[0]
        self.data = X

    def write_file(self,filename):
        with open(filename, "wb") as namdbin:
            np.array(self.n_atoms).astype('int32').tofile(namdbin)
            self.data.astype('float64').tofile(namdbin)

    def getPositions(self):
        return Quantity(self.data, u.angstrom)

    def getVelocities(self):
        # https://www.ks.uiuc.edu/Research/namd/mailing_list/namd-l.2006-2007/0884.html
        PDBVELFACTOR = 20.45482706 
        return Quantity(self.data * PDBVELFACTOR, u.angstrom/u.picosecond)






def remove_barostat(system):
    """Remove MonteCarloBarostat if present"""
    fs = system.getForces()
    for i, f in enumerate(fs):
        if type(f) == simtk.openmm.openmm.MonteCarloBarostat or \
           type(f) == simtk.openmm.openmm.MonteCarloAnisotropicBarostat:
            system.removeForce(i)
            return



# Simple-minded reader for NAMD files

"""Reader for NAMD XSC format."""


import numpy as np
import simtk.unit as u
import simtk.openmm as mm

from simtk.unit.quantity import Quantity




def parse_xsc(xsc):
    with open(xsc) as f:
        for l in f:
            if '#' in l:
                continue
            ls = l.split(" ")
            if len(ls) >= 10:
                boxx = float(ls[1])
                boxy = float(ls[5])
                boxz = float(ls[9])
    return (boxx, boxy, boxz)


def parse_xsc_units(xsc):
    box = parse_xsc(xsc)
    boxa = mm.Vec3(box[0], 0., 0.) * u.angstrom
    boxb = mm.Vec3(0., box[1],  0.) * u.angstrom
    boxc = mm.Vec3(0., 0., box[2]) * u.angstrom
    return (boxa, boxb, boxc)


def write_xsc(xsc, step, stime, np_box):
    import time

    np_box=np_box.value_in_unit(u.angstrom)
    with open(xsc, "w") as f:
        f.write(f"# XSC file generated by miniomm for simulation time {stime} on {time.ctime()}\n")
        f.write("#$LABELS step a_x a_y a_z b_x b_y b_z c_x c_y c_z o_x o_y o_z\n")
        f.write(f"{step} ")
        f.write(" ".join(str(x) for x in np_box[0,:]))
        f.write("  ")
        f.write(" ".join(str(x) for x in np_box[1,:]))
        f.write("  ")
        f.write(" ".join(str(x) for x in np_box[2,:]))
        f.write("  ")
        f.write("0.0 0.0 0.0\n")
    



from simtk.unit import *
from simtk.openmm import *
from simtk.unit import nanometer
from simtk.unit import angstrom
from simtk.unit import picosecond
from simtk.unit import femtosecond


import numpy as np
import time
import math
import os
import sys

# Taken from https://github.com/compsciencelab/MDy/blob/master/python/mdy/reporters.py


class StdoutLogReporter:
    def __init__(self, reportInterval, totalSteps):
        self._reportInterval = reportInterval
        self._inited = False
        self._totalSteps = totalSteps
        self._lastvol = None

    def print_headers(self):
        print("  %10s %10s %11s %11s %11s %8s %8s %8s %8s %11s" % (
            "Step", "Time", "PE", "KE", "Total E", "Temp", "Volume", "Fluct.", "ISpeed", "Completion"))
        print("  %10s %10s %11s %11s %11s %8s %8s %8s %8s %11s" % (
            "", "ns", "kJ/mol", "kJ/mol", "kJ/mol", "K", "nm^3", "%", "ns/day", "dd:hh:mm:ss"))

    def _init(self, simulation, system, state):
        # Compute the number of degrees of freedom.
        dof = 0
        for i in range(system.getNumParticles()):
            if system.getParticleMass(i) > 0 * unit.dalton:
                dof += 3
        dof -= system.getNumConstraints()
        if any(type(system.getForce(i)) == CMMotionRemover for i in range(system.getNumForces())):
            dof -= 3
        self._dof = dof
        self._initialSteps = simulation.currentStep
        self._initialClockTime = time.time()
        self._initialSimulationTime = state.getTime()
        self._lastClockTime = self._initialClockTime
        self._lastSimulationTime = self._initialSimulationTime
        self._inited = True

    def describeNextReport(self, simulation):
        steps = self._reportInterval - simulation.currentStep % self._reportInterval
        return (steps, False, False, False, True)

    def report(self, simulation, state):
        if not self._inited:
            self._init(simulation, simulation.system, state)
            self.print_headers()
        clockTime = time.time()
        step = simulation.currentStep
        timex = state.getTime().value_in_unit(nanosecond)
        pe = state.getPotentialEnergy().value_in_unit(kilojoules_per_mole)
        ke = state.getKineticEnergy().value_in_unit(kilojoules_per_mole)
        te = pe + ke
        temp = (2 * state.getKineticEnergy() / (self._dof *
                                                unit.MOLAR_GAS_CONSTANT_R)).value_in_unit(unit.kelvin)

        speed = -1.0
        elapsedSeconds = clockTime - self._initialClockTime
        if elapsedSeconds > 0:
            elapsedDays = (elapsedSeconds) / 86400.0
            elapsedSteps = simulation.currentStep - self._initialSteps
            elapsedNs = (
                state.getTime() - self._initialSimulationTime).value_in_unit(unit.nanosecond)
            speed = elapsedNs / elapsedDays

        instaSpeed = -1.0
        instaSeconds = clockTime - self._lastClockTime
        if instaSeconds > 0:
            instaDays = instaSeconds / 86400.0
            instaNs = (state.getTime() -
                       self._lastSimulationTime).value_in_unit(unit.nanosecond)
            instaSpeed = instaNs / instaDays
            self._lastClockTime = clockTime
            self._lastSimulationTime = state.getTime()

        if elapsedSteps:
            estimatedTotalSeconds = (
                self._totalSteps - self._initialSteps) * elapsedSeconds / elapsedSteps
        else:
            estimatedTotalSeconds = 0
        remainingSeconds = int(estimatedTotalSeconds - elapsedSeconds)
        remainingDays = remainingSeconds // 86400
        remainingSeconds -= remainingDays * 86400
        remainingHours = remainingSeconds // 3600
        remainingSeconds -= remainingHours * 3600
        remainingMinutes = remainingSeconds // 60
        remainingSeconds -= remainingMinutes * 60
        remianingString = "--"
        if remainingDays > 0:
            remainingString = "%d:%d:%02d:%02d" % (
                remainingDays, remainingHours, remainingMinutes, remainingSeconds)
        elif remainingHours > 0:
            remainingString = "%d:%02d:%02d" % (
                remainingHours, remainingMinutes, remainingSeconds)
        elif remainingMinutes > 0:
            remainingString = "%d:%02d" % (remainingMinutes, remainingSeconds)
        else:
            remainingString = "0:%02d" % remainingSeconds

        box = state.getPeriodicBoxVectors()
        v = box[0][0] * box[1][1] * box[2][2]
        volume = v.value_in_unit(nanometer ** 3)
        if self._lastvol:
            fluctuation = 100. * (volume / self._lastvol - 1.)
        else:
            fluctuation = 0.
        self._lastvol = volume

        print("# %10ld %10.3f %11.2f %11.2f %11.2f %8.2f %8.2f %8.2f %8.2f %11s" % (
            step, timex, pe, ke, te, temp, volume, fluctuation, instaSpeed, remainingString))

        if (math.isnan(pe) or math.isnan(ke) or math.isnan(temp)):
            raise ValueError("Simulation has become unstable. Aborted")





import sys
import os.path

from simtk.openmm import app
import simtk.openmm as mm
import simtk.unit as u

from miniomm.config import Config
import miniomm.util as util
from miniomm.namdbin import NAMDBin
from miniomm.namdxsc import write_xsc


# Order of priority for the operation to perform
#   Minimization if minim.coor absent
#   NPT if equilib.coor absent
#   NVT otherwise

# Order of priority for box size
#   Checkpoint file
#   Box size (input.xsc)

# Order of priority for starting coordinates
#   Checkpoint file
#   Input coordinates (input.coor)
#   PDB coordinates (structure.pdb)


# Order of priority for starting velocities
#   Checkpoint file
#   Input velocities (input.vel)
#   Randomized velocities

checkpoint_file = "miniomm_restart.chk"


def run_omm(options):

    inp = Config(options.input)

    dt = float(inp.timestep) * u.femtosecond
    temperature = float(inp.temperature) * u.kelvin
    logPeriod = 1 * u.picosecond
    trajectoryPeriod = int(inp.trajectoryperiod) * dt
    run_steps = int(inp.run)
    basename = "output"
    trajectory_file = inp.trajectoryfile.replace(".xtc", ".dcd")

    if 'PME' in inp and inp.getboolean('PME'):
        nonbondedMethod = app.PME
    else:
        nonbondedMethod = app.NoCutoff
    nonbondedCutoff = float(inp.cutoff) * u.angstrom
    switchDistance = float(inp.switchdistance) * u.angstrom
    frictionCoefficient = float(inp.thermostatdamping) / u.picosecond

    endTime = run_steps * dt


    util.check_openmm()

    if options.platform is None:
        print("Selecting best platform:")
        req_platform_name = util.get_best_platform()
    else:
        print(f"Requesting platform {options.platform}")
        req_platform_name = options.platform
    req_platform = mm.Platform.getPlatformByName(req_platform_name)

    req_properties = {}    # {'UseBlockingSync':'true'}
    if options.device is not None and 'DeviceIndex' in req_platform.getPropertyNames():
        print("    Setting DeviceIndex = "+options.device)
        req_properties['DeviceIndex'] = options.device
    if options.precision is not None and 'Precision' in req_platform.getPropertyNames():
        print("    Setting Precision = "+options.precision)
        req_properties['Precision'] = options.precision

    # Same logic as https://software.acellera.com/docs/latest/acemd3/reference.html
    if dt > 2 * u.femtosecond:
        hydrogenMass = 4 * u.amu
        constraints = app.AllBonds
        rigidWater = True
    elif dt > 0.5 * u.femtosecond:
        hydrogenMass = None
        constraints = app.HBonds
        rigidWater = True
    else:
        hydrogenMass = None
        constraints = None
        rigidWater = False


    print("")
    print(f"Hydrogen mass repartitioning: {hydrogenMass}     constraints: {constraints}\n"+
          f"                 rigid water: {rigidWater}     nonbonded:   {nonbondedMethod}")
    print("")

    if 'parmfile' in inp: 
        print(f"Creating an AMBER system...")
        if 'structure' in inp:
            print("Warning: 'structure' given but ignored for AMBER")
        prmtop = app.AmberPrmtopFile(inp.parmfile)
        system = prmtop.createSystem(nonbondedMethod=nonbondedMethod,
                                     nonbondedCutoff=nonbondedCutoff,
                                     switchDistance=switchDistance,
                                     constraints=constraints,
                                     hydrogenMass=hydrogenMass,
                                     rigidWater=rigidWater)
        topology = prmtop.topology
    else:
        print(f"Creating a CHARMM system...")
        psf = app.CharmmPsfFile(inp.structure)
        params = app.CharmmParameterSet(inp.parameters, permissive=True)
        psf.setBox( 50.*u.angstrom, 50.*u.angstrom, 50.*u.angstrom) # otherwise
                                                                    # refuses
                                                                    # PME
        system = psf.createSystem(params,
                                  nonbondedMethod=nonbondedMethod,
                                  nonbondedCutoff=nonbondedCutoff,
                                  switchDistance=switchDistance,
                                  constraints=constraints,
                                  hydrogenMass=hydrogenMass,
                                  rigidWater=rigidWater)
        topology = psf.topology

    if 'barostat' in inp and inp.getboolean('barostat'):
        pressure = float(inp.barostatpressure) * u.bar
        print(f"Enabling barostat at {pressure}...")
        system.addForce(mm.MonteCarloBarostat(pressure, temperature))

    if 'plumedfile' in inp:
        print("Attempting to load PLUMED plugin...")
        from openmmplumed import PlumedForce
        plines = util.plumed_parser(inp.plumedfile)
        system.addForce(PlumedForce(plines))

    integrator = mm.LangevinIntegrator(temperature,
                                       frictionCoefficient,
                                       dt)
    integrator.setConstraintTolerance(1e-5)

    simulation = app.Simulation(topology, system, integrator,
                                req_platform, req_properties)
    ctx = simulation.context
    platform = ctx.getPlatform()
    print(f"Got platform {platform.getName()} with properties:")
    for prop in platform.getPropertyNames():
        print(f"    {prop}\t\t{platform.getPropertyValue(ctx,prop)}")
    print("")

    resuming = False
    if os.path.exists(checkpoint_file):
        with open(checkpoint_file, 'rb') as cf:
            ctx.loadCheckpoint(cf.read())
        # ctx.loadCheckpoint(str(checkpoint_file))
        print(f"Successfully loaded {checkpoint_file}, resuming simulation...")
        resuming = True

    else:
        print(f"File {checkpoint_file} absent, starting simulation from the beginning...")
        coords = util.get_coords(inp)
        ctx.setPositions(coords)

    if not resuming:
        (boxa, boxb, boxc) = util.get_box_size(inp)
        ctx.setPeriodicBoxVectors(boxa, boxb, boxc)

        if 'minimize' in inp:
            print(f'Minimizing for max {inp.minimize} iterations...')
            simulation.minimizeEnergy(maxIterations=int(inp.minimize))
            simulation.saveState(f"miniomm_minimized.xml")
        else:
            if 'binvelocities' in inp:
                print(f"Reading velocities from NAMDBin: "+inp.binvelocities)
                vels = NAMDBin(inp.binvelocities).getVelocities()
                ctx.setVelocities(vels)
            else:
                print(f"Resetting thermal velocities at {temperature}")
                ctx.setVelocitiesToTemperature(temperature)

    # -------------------------------------------------------
    print("")

    startTime = ctx.getState().getTime()
    startTime_f = startTime.in_units_of(u.nanoseconds).format("%.3f")
    endTime_f = endTime.in_units_of(u.nanoseconds).format("%.3f")
    remaining_steps = round((endTime-startTime)/dt)
    remaining_ns = remaining_steps * dt.value_in_unit(u.nanosecond)
    print(f"Current simulation time is {startTime_f}, running up to {endTime_f}.")
    print(f"Will run for {remaining_steps} timesteps = {remaining_ns:.3f} ns...")
    print("")

    log_every = util.every(logPeriod, dt)
    save_every = util.every(trajectoryPeriod, dt)
    if remaining_steps % save_every != 0:
        raise ValueError("Remaining steps is not a multiple of trajectoryperiod")

    util.add_reporters(simulation, trajectory_file,
                       log_every, save_every,
                       remaining_steps, resuming, checkpoint_file)

    # ----------------------------------------
    simulation.saveState(f"miniomm_pre.xml")

    # ----------------------------------------
    simulation.step(remaining_steps)

    # ----------------------------------------
    simulation.saveState(f"miniomm_post.xml")
    final_state = simulation.context.getState(getPositions=True,
                                              getVelocities=True)
    final_coor = final_state.getPositions(asNumpy=True)
    NAMDBin(final_coor).write_file(f"{basename}.coor")

    final_box = final_state.getPeriodicBoxVectors(asNumpy=True)
    write_xsc(f"{basename}.xsc", remaining_steps, final_state.getTime(), final_box)
    # ----------------------------------------
    print('Done!')
    return




def main():
    from optparse import OptionParser
    parser = OptionParser()
    platformNames = [mm.Platform.getPlatform(
        i).getName() for i in range(mm.Platform.getNumPlatforms())]
    parser.add_option('--input', dest='input',
                      default="input", help='name of the input file')
    parser.add_option('--platform', dest='platform',
                      choices=platformNames, help='name of the platform to benchmark')
    parser.add_option('--device', default=None, dest='device',
                      help='device index for CUDA or OpenCL')
    parser.add_option('--precision', dest='precision', choices=('single', 'mixed', 'double'),
                      help='precision mode for CUDA or OpenCL: single, mixed, or double')
    parser.add_option('--hours', default='11.5', dest='run_hours', type='float',
                      help='target simulation length in hours [default: 11.5]')

    (options, args) = parser.parse_args()

    if len(args) > 0:
        print("Remaining args: "+" ".join(args))

    print(util.getBanner())
    run_omm(options)


if __name__ == "__main__":
    main()
    



import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "miniomm",
    version = "0.0.12",
    author = "Toni Giorgino",
    author_email = "toni.giorgino@gmail.com",
    description = ("A simple OpenMM wrapper for common-case MD runs"),
    license = "LGPL",
    keywords = "molecular dynamics",
    url = "https://github.com/giorginolab/miniomm",
    packages=['miniomm', 'tests'],
    entry_points = {
        'console_scripts': ['miniomm=miniomm.miniomm:main']
    },
    scripts = ['bin/sbatch_many'],
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Physics",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)"
    ],
    # install_requires = [ 'simtk.openmm', 'simtk.unit' ],
)



import configparser
from collections import OrderedDict

root_section = "ROOT"


class MultiOrderedDict(OrderedDict):
    def __setitem__(self, key, value):
        if isinstance(value, list) and key in self:
            self[key].extend(value)
        else:
            # super(MultiOrderedDict, self).__setitem__(key, value)
            super().__setitem__(key, value)


class Config():
    def __init__(self, fn):
        self.cp = configparser.RawConfigParser(dict_type=MultiOrderedDict,
                                               delimiters=[' ', "\t", '='],
                                               strict=False)
        with open(fn) as f:
            self.cp.read_string('['+root_section+"]\n"+f.read())

    def get(self, k):
        return self.cp.get(root_section, k)

    def __getattr__(self, k):
        return self.get(k)

    def __contains__(self, k):
        return k in self.cp[root_section]

    def getboolean(self, k):
        v = self.__getattr__(k).lower()
        return (v == "on" or v == "true")


if __name__ == "__main__":
    import sys

    c = Config("scratch/pdir_amber/input")
    c.cp.write(sys.stdout)
